﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Programmable {
  [Item("ProblemDefinition", "Base class for problem definitions.")]
  [StorableClass]
  public abstract class ProblemDefinition : NamedItem, IProblemDefinition {
    [Storable(Name = "Encoding")]
    private IEncoding encoding;
    public IEncoding Encoding {
      get { return encoding; }
      protected set {
        if (value == null) throw new ArgumentNullException("The encoding must not be null.");
        if (encoding == value) return;
        encoding = value;
        OnProblemDefinitionChanged();
      }
    }

    [StorableConstructor]
    protected ProblemDefinition(bool deserializing) : base(deserializing) { }
    protected ProblemDefinition(ProblemDefinition original, Cloner cloner)
      : base(original, cloner) {
      this.encoding = cloner.Clone(original.encoding);
    }
    protected ProblemDefinition(IEncoding encoding) : this(encoding, "ProblemDefinition") { }
    protected ProblemDefinition(IEncoding encoding, string name) : this(encoding, name, string.Empty) { }
    protected ProblemDefinition(IEncoding encoding, string name, string description)
      : base(name, description) {
      Encoding = encoding;
    }

    public event EventHandler ProblemDefinitionChanged;
    protected void OnProblemDefinitionChanged() {
      var handler = ProblemDefinitionChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    public abstract IEnumerable<Individual> GetNeighbors(IRandom random, Individual vector);
  }
}
